"""
    Patch a given product, or list of products as specified.
"""
# $Id: Patch.py,v 1.2 2001/10/21 09:58:19 adrian Exp $
#
# $Log: Patch.py,v $
# Revision 1.2  2001/10/21 09:58:19  adrian
# Patched to prevent multiple management tabs from appearing for patches that create multiple tabs.
#
#
__version__ = '$Revision: 1.2 $'[11:-2]

import Globals
import Acquisition

def applyPatch(module):
    """ Given a patch module (Not the NAME, the MODULE)
        apply the defined patches

        Patch modules must contain at least:
         .targets             (List/Tuple of products to patch)
         .manage_options      (List/Tuple of manage_options to add)
         .backup_methods      (Dictionary mapping new names to old) (Optional)
         .methods             (Dictionary mapping names to methods)
    """
  
    if not hasattr(module, 'targets'):
        module.targets = []

    if not hasattr(module, 'backup_methods'):
        module.backup_methods = {}

    if not hasattr(module, 'methods'):
        module.methods = {}

    if type(module.backup_methods)!=type({}):
        raise "Patch Error", "'backup_methods' bad in module: %s" % module.__name__

    if type(module.methods)!=type({}):
        raise "Patch Error", "'methods' bad in module: %s" % module.__name__

    for target in module.targets:
    
        for method in module.backup_methods.items():
            if not hasattr(target, method[1]):
                setattr(target, method[1], getattr(target, method[0]))
                
        for method in module.methods.items():
            setattr(target, method[0], method[1])
            
        newOptions = []
        if hasattr(module, 'manage_options') and hasattr(Acquisition.aq_base(target), 'manage_options'):
            labels = []
            for option in module.manage_options:
                labels.append(option['label'])
                
            for option in target.manage_options:
                if not option.has_key('label') or not option['label'] in labels:
                    newOptions.append(option)
                    
            for newMo in module.manage_options:
                newOptions.append(newMo)
                
            setattr(target, 'manage_options', tuple(newOptions))
            
        Globals.default__class_init__(target)
